@@ -41,6 +41,7 @@ class Germinator:
         self.seeds = []
         self.seed = {}
         self.seedinherit = {}
+        self.seedrelease = {}
         self.substvars = {}
         self.depends = {}
         self.build_depends = {}
@@ -218,10 +219,11 @@ class Germinator:
             fh.write('%s\t%s\n' % (pkg, blacklist))
         fh.close()
 
-    def newSeed(self, seedname, seedinherit):
+    def newSeed(self, seedname, seedinherit, seedrelease):
         self.seeds.append(seedname)
         self.seed[seedname] = []
         self.seedinherit[seedname] = seedinherit
+        self.seedrelease[seedname] = seedrelease
         self.depends[seedname] = []
         self.build_depends[seedname] = []
         self.sourcepkgs[seedname] = []
@@ -294,12 +296,12 @@ class Germinator:
 
         return False
 
-    def plantSeed(self, entries, arch, seedname, seedinherit):
+    def plantSeed(self, entries, arch, seedname, seedinherit, seedrelease=None):
         """Add a seed."""
         if seedname in self.seeds:
             return
 
-        self.newSeed(seedname, seedinherit)
+        self.newSeed(seedname, seedinherit, seedrelease)
         seedpkgs = []
 
         for line in entries:
@@ -324,21 +326,35 @@ class Germinator:
                 self.substvars[name] = values
                 continue
 
+            pkg = pkg.strip()
             archspec = []
             startarchspec = pkg.find("[")
             if startarchspec != -1:
                 endarchspec = pkg.find("]")
-                if endarchspec == -1:
-                    self.error("Broken architecture specification: %s", pkg)
-                else:
-                    archspec = pkg[startarchspec + 1:endarchspec].split()
+                if pkg.endswith("]"):
+                    archspec = pkg[startarchspec + 1:-1].split()
                     pkg = pkg[:startarchspec - 1]
                     if arch not in archspec:
                         continue
 
             pkg = pkg.split()[0]
 
-            seedpkgs.extend(self.substituteSeedVars(pkg))
+            if pkg.startswith('%'):
+                pkg = pkg[1:]
+                if pkg in self.sources:
+                    pkgs = list(self.sources[pkg]["Binaries"])
+                else:
+                    self.warning("Unknown source package: %s", pkg)
+                    pkgs = []
+            elif pkg.startswith('/') and pkg.endswith('/'):
+                pkgre = re.compile(pkg[1:-1])
+                pkgs = [p for p in self.packages if pkgre.search(p) is not None]
+                pkgs.sort()
+            else:
+                pkgs = [pkg]
+
+            for pkg in pkgs:
+                seedpkgs.extend(self.substituteSeedVars(pkg))
 
         for pkg in seedpkgs:
             if pkg in self.hints and self.hints[pkg] != seedname:
@@ -392,17 +408,24 @@ class Germinator:
         for seedname in self.seeds:
             self.progress("Resolving %s dependencies ...", seedname)
             for pkg in self.seed[seedname]:
-                self.addPackage(seedname, pkg, seedname.title() + " seed")
+                if self.seedrelease[seedname] is None:
+                    self.addPackage(seedname, pkg, "%s seed" %
+                        seedname.title())
+                else:
+                    self.addPackage(seedname, pkg, "%s %s seed" %
+                        (self.seedrelease[seedname].title(), seedname))
 
-    def addExtras(self):
+    def addExtras(self, seedrelease=None):
         """Add packages generated by the sources but not in any seed."""
-        self.newSeed("extra", self.seeds)
+        self.newSeed("extra", self.seeds, seedrelease)
 
         self.progress("Identifying extras ...")
         for srcname in self.all_srcs:
             for pkg in self.sources[srcname]["Binaries"]:
                 if pkg not in self.packages:
                     continue
+                if self.packages[pkg]["Source"] != srcname:
+                    continue
                 if pkg in self.all:
                     continue
 
@@ -436,6 +459,17 @@ class Germinator:
             else:
                 return False
 
+    def allowedVirtualDependency(self, pkg, deptype):
+        """May pkg's dependency relationship type deptype be satisfied by a
+           virtual package? (Versioned dependencies may not be satisfied by
+           virtual packages, unless pkg is a udeb.)"""
+        if pkg in self.packagetype and self.packagetype[pkg] == "udeb":
+            return True
+        elif deptype == "":
+            return True
+        else:
+            return False
+
     def addReverse(self, pkg, field, rdep):
         """Add a reverse dependency entry."""
         if "Reverse-Depends" not in self.packages[pkg]:
@@ -476,12 +510,13 @@ class Germinator:
 
     def alreadySatisfied(self, seedname, pkg, depend, build_depend=False, with_build=False):
         """Work out whether a dependency has already been satisfied."""
-        if depend in self.provides:
-            trylist = [ d for d in self.provides[depend]
+        (depname, depver, deptype) = depend
+        if self.allowedVirtualDependency(pkg, deptype) and depname in self.provides:
+            trylist = [ d for d in self.provides[depname]
                         if d in self.packages and self.allowedDependency(pkg, d, seedname, build_depend) ]
-        elif depend in self.packages and \
-             self.allowedDependency(pkg, depend, seedname, build_depend):
-            trylist = [ depend ]
+        elif depname in self.packages and \
+             self.allowedDependency(pkg, depname, seedname, build_depend):
+            trylist = [ depname ]
         else:
             return False
 
@@ -503,16 +538,17 @@ class Germinator:
                       second_class, build_tree):
         """Add a single dependency. Returns True if a dependency was added,
            otherwise False."""
-        if depend in self.packages and \
-           self.allowedDependency(pkg, depend, seedname, build_depend):
+        (depname, depver, deptype) = depend
+        if depname in self.packages and \
+           self.allowedDependency(pkg, depname, seedname, build_depend):
             virtual = None
-            trylist = [ depend ]
-        elif depend in self.provides:
-            virtual = depend
-            trylist = [ d for d in self.provides[depend]
+            trylist = [ depname ]
+        elif self.allowedVirtualDependency(pkg, deptype) and depname in self.provides:
+            virtual = depname
+            trylist = [ d for d in self.provides[depname]
                         if d in self.packages and self.allowedDependency(pkg, d, seedname, build_depend) ]
         else:
-            self.error("Unknown dependency %s by %s", depend, pkg)
+            self.error("Unknown dependency %s by %s", depname, pkg)
             return False
 
         # Last ditch effort to satisfy this by promoting lesser seeds to
@@ -522,30 +558,38 @@ class Germinator:
             for lesserseed in self.strictlyOuterSeeds(seedname):
                 if trydep in self.seed[lesserseed]:
                     if second_class:
-                        # I'll get you next time, Gadget!
-                        return False
-                    self.seed[lesserseed].remove(trydep)
-                    self.warning("Promoted %s from %s to %s to satisfy %s",
-                                 trydep, lesserseed, seedname, pkg)
+                        # "I'll get you next time, Gadget!"
+                        # When processing the build tree, we don't promote
+                        # packages from lesser seeds, since we still want to
+                        # consider them (e.g.) part of ship even if they're
+                        # build-dependencies of desktop. However, we do need
+                        # to process them now anyway, since otherwise we
+                        # might end up selecting the wrong alternative from
+                        # an or-ed build-dependency.
+                        pass
+                    else:
+                        self.seed[lesserseed].remove(trydep)
+                        self.warning("Promoted %s from %s to %s to satisfy %s",
+                                     trydep, lesserseed, seedname, pkg)
 
-                    depend = trydep
+                    depname = trydep
                     found = True
                     break
             if found: break
 
-        dependlist = [depend]
+        dependlist = [depname]
         if virtual is not None and not found:
             reallist = [ d for d in self.provides[virtual]
                          if d in self.packages and self.allowedDependency(pkg, d, seedname, build_depend) ]
             if len(reallist):
-                depend = reallist[0]
+                depname = reallist[0]
                 # If this one was a d-i kernel module, pick all the modules
                 # for other allowed kernel versions too.
-                if self.packages[depend]["Kernel-Version"] != "":
+                if self.packages[depname]["Kernel-Version"] != "":
                     dependlist = [ d for d in reallist
                                    if self.packages[d]["Kernel-Version"] in self.di_kernel_versions[seedname] ]
                 else:
-                    dependlist = [depend]
+                    dependlist = [depname]
                 self.info("Chose %s out of %s to satisfy %s",
                           ", ".join(dependlist), virtual, pkg)
             else:
@@ -579,11 +623,11 @@ class Germinator:
         if build_tree: second_class = True
         for deplist in depends:
             for dep in deplist:
-                if self.alreadySatisfied(seedname, pkg, dep[0], build_depend, second_class):
+                if self.alreadySatisfied(seedname, pkg, dep, build_depend, second_class):
                     break
             else:
                 for dep in deplist:
-                    if self.addDependency(seedname, pkg, dep[0], build_depend,
+                    if self.addDependency(seedname, pkg, dep, build_depend,
                                           second_class, build_tree):
                         if len(deplist) > 1:
                             self.info("Chose %s to satisfy %s", dep[0], pkg)
