@@ -36,13 +36,14 @@ import Germinate.Archive
 
 # Where do we get up-to-date seeds from?
 SEEDS = "http://people.ubuntu.com/~cjwatson/seeds/"
-RELEASE = "breezy"
+RELEASE = "dapper"
 
 # If we need to download Packages.gz and/or Sources.gz, where do we get
 # them from?
 MIRROR = "http://archive.ubuntu.com/ubuntu/"
-DIST = ["breezy"]
-COMPONENTS = ["main"]
+SOURCE_MIRROR = None
+DIST = ["dapper"]
+COMPONENTS = ["main", "restricted"]
 ARCH = "i386"
 
 CHECK_IPV6 = False
@@ -253,11 +254,15 @@ Options:
   -s, --seed-dist=DIST  Fetch seeds for distribution DIST (default: %s).
   -m, --mirror=MIRROR   Get package lists from MIRROR
                         (default: %s).
+  --source-mirror=MIRROR
+                        Get source package lists from mirror
+                        (default: value of --mirror).
   -d, --dist=DIST       Operate on distribution DIST (default: %s).
   -a, --arch=ARCH       Operate on architecture ARCH (default: %s).
   -c, --components=COMPS
                         Operate on components COMPS (default: %s).
   -i, --ipv6            Check IPv6 status of source packages.
+  --cleanup             Don't cache Packages or Sources files.
   --no-rdepends         Disable reverse-dependency calculations.
   --seed-packages=PARENT/PKG,PARENT/PKG,...
                         Treat each PKG as a seed by itself, inheriting from
@@ -266,7 +271,9 @@ Options:
 
 
 def main():
-    global SEEDS, RELEASE, MIRROR, DIST, ARCH, COMPONENTS, CHECK_IPV6
+    global SEEDS, RELEASE, MIRROR, SOURCE_MIRROR
+    global DIST, ARCH, COMPONENTS, CHECK_IPV6
+    cleanup = False
     want_rdepends = True
     seed_packages = ()
 
@@ -284,10 +291,12 @@ def main():
                                     "seed-source=",
                                     "seed-dist=",
                                     "mirror=",
+                                    "source-mirror=",
                                     "dist=",
                                     "components=",
                                     "arch=",
                                     "ipv6",
+                                    "cleanup",
                                     "no-rdepends",
                                     "seed-packages="])
     except getopt.GetoptError:
@@ -308,6 +317,10 @@ def main():
             MIRROR = value
             if not MIRROR.endswith("/"):
                 MIRROR += "/"
+        elif option == "--source-mirror":
+            SOURCE_MIRROR = value
+            if not SOURCE_MIRROR.endswith("/"):
+                SOURCE_MIRROR += "/"
         elif option in ("-d", "--dist"):
             DIST = value.split(",")
         elif option in ("-c", "--components"):
@@ -316,6 +329,8 @@ def main():
             ARCH = value
         elif option in ("-i", "--ipv6"):
             CHECK_IPV6 = True
+        elif option == "--cleanup":
+            cleanup = True
         elif option == "--no-rdepends":
             want_rdepends = False
         elif option == "--seed-packages":
@@ -324,7 +339,8 @@ def main():
     apt_pkg.InitConfig()
     apt_pkg.Config.Set("APT::Architecture", ARCH)
 
-    Germinate.Archive.TagFile(MIRROR).feed(g, DIST, COMPONENTS, ARCH)
+    Germinate.Archive.TagFile(MIRROR, SOURCE_MIRROR).feed(
+        g, DIST, COMPONENTS, ARCH, cleanup)
 
     if CHECK_IPV6:
         g.parseIPv6(open_ipv6_tag_file("dailydump"))
@@ -339,14 +355,15 @@ def main():
     (seednames, seedinherit) = g.parseStructure(open_metafile("STRUCTURE"))
     for seedname in seednames:
         g.plantSeed(open_metafile(seedname), ARCH, seedname,
-                    list(seedinherit[seedname]))
+                    list(seedinherit[seedname]), RELEASE)
     for seed_package in seed_packages:
         (parent, pkg) = seed_package.split('/')
-        g.plantSeed([" * " + pkg], ARCH, pkg, seedinherit[parent] + [parent])
+        g.plantSeed([" * " + pkg], ARCH, pkg,
+                    seedinherit[parent] + [parent], RELEASE)
         seednames.append(pkg)
     g.prune()
     g.grow()
-    g.addExtras()
+    g.addExtras(RELEASE)
     if want_rdepends:
         g.reverseDepends()
 
